/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 * 
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.expression.lib;

import org.objectweb.jorm.type.api.PType;
import org.objectweb.medor.expression.api.ExpressionException;
import org.objectweb.medor.expression.api.MalformedExpressionException;
import org.objectweb.medor.expression.api.Operand;
import org.objectweb.medor.expression.api.ParameterOperand;
import org.objectweb.medor.expression.api.TypingException;
import org.objectweb.medor.expression.api.UnaryArithmeticOperator;
import org.objectweb.medor.expression.api.Expression;
import org.objectweb.medor.expression.type.ExpressionTypeHelper;

/**
 *
 * @author Sebastien Chassande-Barrioz
 */
public abstract class BasicUnaryArithmeticOperator
        extends BasicUnaryOperator
        implements UnaryArithmeticOperator {

	public BasicUnaryArithmeticOperator() {
	}

	public BasicUnaryArithmeticOperator(PType type) {
		super(type);
	}

	public BasicUnaryArithmeticOperator(Expression e) {
		super(e);
	}

	public BasicUnaryArithmeticOperator(PType type, Expression e) {
		super(type, e);
	}

    // IMPLEMENTATION OF THE Expression INTERFACE //
    //--------------------------------------------//

    public org.objectweb.medor.expression.api.Operand evaluate(ParameterOperand[] pos, Object o)
            throws ExpressionException {
        try {
            Operand subResult = expressions[0].evaluate(pos, o);
            switch (subResult.getType().getTypeCode()) {
                case PType.TYPECODE_DOUBLE:
                    result.setValue(evaluate(subResult.getDouble()));
                    break;
                case PType.TYPECODE_FLOAT:
                    result.setValue(evaluate(subResult.getFloat()));
                    break;
                case PType.TYPECODE_BYTE:
                    result.setValue(evaluate(subResult.getByte()));
                    break;
                case PType.TYPECODE_SHORT:
                    result.setValue(evaluate(subResult.getShort()));
                    break;
                case PType.TYPECODE_INT:
                    result.setValue(evaluate(subResult.getInt()));
                    break;
                case PType.TYPECODE_LONG:
                    result.setValue(evaluate(subResult.getLong()));
                    break;
                default:
                    throw new MalformedExpressionException(
                            "Unauthorized expression element");
            }
        } catch (NullPointerException e) {
            throw new IllegalStateException("Unevaluable Expression: Not compiled");
        }
		return result;
    }

    public Operand compileExpression()
		throws ExpressionException, MalformedExpressionException {
        if (expressions[0] != null) {
            expressions[0].compileExpression();
            if (ExpressionTypeHelper.isArithmeticType(expressions[0].getType())) {
                type = expressions[0].getType();
                result = new BasicVariableOperand(type);
                verified = true;
            } else
                throw new TypingException("Attempt an aritmetic type");
        } else
            throw new MalformedExpressionException("null childeren value");// Operands not yet assigned
		return result;
    }
}
