/**
 * Copyright (C) 2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.medor.clone;

import org.objectweb.medor.filter.TestExpressionHelper;
import org.objectweb.medor.clone.lib.BasicCloneable;
import org.objectweb.medor.expression.api.*;
import org.objectweb.medor.expression.lib.*;

import org.objectweb.jorm.type.api.PTypeSpace;

/**
 *
 * @author S.Chassande-Barrioz
 */
public class TestExpressionCloning extends TestExpressionHelper {

	public TestExpressionCloning(String n) {
		super(n);
	}

	private Object checkClone(Expression obj, boolean withContext) throws Exception {
		Object clone = (withContext
			? ((BasicCloneable) obj).clone()
			: ((BasicCloneable) obj).clone(null));
		assertNotNull("Null clone", clone);
		assertEquals("Bad class", obj.getClass(), clone.getClass());
		assertEquals("Bad type", obj.getType(), ((Expression) clone).getType());
		return clone;
	}

	public void testOperandWithoutContext() throws Exception {
		testOperand(false);
	}

	public void testOperandWithContext() throws Exception {
		testOperand(true);
	}

	public void testOperand(boolean withContext) throws Exception {
		BasicOperand bo;
		Object clone;

		//BasicOperand
		bo = new BasicOperand(true);
		clone = checkClone(bo, withContext);
		assertEquals("Bad operand value", bo.getBoolean(), ((BasicOperand) clone).getBoolean());

		bo = new BasicOperand(12);
		clone = checkClone(bo, withContext);
		assertEquals("Bad operand value", bo.getInt(), ((BasicOperand) clone).getInt());

		bo = new BasicOperand("azerty");
		clone = checkClone(bo, withContext);
		assertEquals("Bad operand value", bo.getString(), ((BasicOperand) clone).getString());

		//BasicVariableOperand
		bo = new BasicVariableOperand(true);
		clone = checkClone(bo, withContext);
		assertEquals("Bad operand value", bo.getBoolean(), ((BasicOperand) clone).getBoolean());

		bo = new BasicVariableOperand(12);
		clone = checkClone(bo, withContext);
		assertEquals("Bad operand value", bo.getInt(), ((BasicOperand) clone).getInt());

		bo = new BasicVariableOperand("azerty");
		clone = checkClone(bo, withContext);
		assertEquals("Bad operand value", bo.getString(), ((BasicOperand) clone).getString());

		//BasicParameterOperand
		bo = new BasicParameterOperand(PTypeSpace.BOOLEAN, "p1", true);
		clone = checkClone(bo, withContext);
		assertEquals("Bad operand value", bo.getBoolean(), ((BasicOperand) clone).getBoolean());

		bo = new BasicParameterOperand(PTypeSpace.INT, "p2", 12);
		clone = checkClone(bo, withContext);
		assertEquals("Bad operand value", bo.getInt(), ((BasicOperand) clone).getInt());

		bo = new BasicParameterOperand(PTypeSpace.STRING, "p3", "azerty");
		clone = checkClone(bo, withContext);
		assertEquals("Bad operand value", bo.getString(), ((BasicOperand) clone).getString());
	}

	public void testLogicalOperatorwithoutContext() throws Exception {
		testLogicalOperator(false);
	}
	public void testLogicalOperatorwithContext() throws Exception {
		testLogicalOperator(true);
	}
	public void testLogicalOperator(boolean withContext) throws Exception {
		Object clone;
		Expression e = new And(
			new Not(new BasicOperand(true)),
			new Or(
				new Equal(new BasicOperand(12), new BasicOperand(13)),
				new Greater(
					new BasicParameterOperand(PTypeSpace.BOOLEAN, "p1", true),
					new Length(
						new Concat(
							new StringLower(new BasicOperand("TOTO")),
							new Like(
								new BasicOperand("hjklm"),
								new BasicOperand("M*"),
								new BasicOperand("k")
							)
						)
					)
				)
			)
		);
		clone = checkClone(e, withContext);
		equals("testLogicalOperator(" + withContext +"):",
			e, (Expression) clone, logger);
	}

	public void testArithmeticOperatorwithoutContext() throws Exception {
		testLogicalOperator(false);
	}
	public void testArithmeticOperatorwithContext() throws Exception {
		testLogicalOperator(true);
	}
	public void testArithmeticOperator(boolean withContext) throws Exception {
		Object clone;
		Expression e = new Mult(
				new Mod(
					new BasicOperand(24),
					new BasicOperand(24)
				),
				new DivideBy(
					new Plus(new Abs(new BasicOperand(-12)), new BasicOperand(13)),
					new BasicParameterOperand(PTypeSpace.INT, "p1", 45)
				)
		);
		clone = checkClone(e, withContext);
		equals("testArithmeticOperator(" + withContext +"):",
			e, (Expression) clone, logger);
	}
}
